#ifndef beamShiftAXIS_H
#define beamShiftAXIS_H
#include "turboPmacAxis.h"
#include "turboPmacController.h"

class beamShiftAxis : public turboPmacAxis {
  public:
    /**
     * @brief Construct a new beamShiftAxis
     *
     * @param pController         Pointer to the associated controller
     * @param axisNo              Index of the axis
     */
    beamShiftAxis(turboPmacController *pController, int axisNo);

    /**
     * @brief Destroy the turboPmacAxis
     *
     */
    virtual ~beamShiftAxis();

    /**
     * @brief Implementation of the `stop` function from asynMotorAxis
     *
     * @param acceleration        Acceleration ACCEL from the motor record. This
     * value is currently not used.
     * @return asynStatus
     */
    asynStatus stop(double acceleration);

    /**
     * @brief Implementation of the `doHome` function from sinqAxis. The
     * parameters are described in the documentation of `sinqAxis::doHome`.
     *
     * @param minVelocity
     * @param maxVelocity
     * @param acceleration
     * @param forwards
     * @return asynStatus
     */
    asynStatus doHome(double minVelocity, double maxVelocity,
                      double acceleration, int forwards);

    /**
     * @brief Implementation of the `doPoll` function from sinqAxis. The
     * parameters are described in the documentation of `sinqAxis::doPoll`.
     *
     * @param moving
     * @return asynStatus
     */
    asynStatus doPoll(bool *moving);

    /**
     * @brief Implementation of the `doMove` function from sinqAxis. The
     * parameters are described in the documentation of `sinqAxis::doMove`.
     *
     * @param position
     * @param relative
     * @param min_velocity
     * @param max_velocity
     * @param acceleration
     * @return asynStatus
     */
    asynStatus doMove(double position, int relative, double min_velocity,
                      double max_velocity, double acceleration);

    /**
     * @brief Readout of some values from the controller at IOC startup
     *
     * The following steps are performed:
     * - Read out the motor status, motor position, velocity and acceleration
     * from the MCU and store this information in the parameter library.
     * - Set the enable PV according to the initial status of the axis.
     *
     * @return asynStatus
     */
    asynStatus init();

    /**
     * @brief Implementation of the `doReset` function from sinqAxis.
     *
     * @param on
     * @return asynStatus
     */
    asynStatus doReset();

    /**
     * @brief Enable the axis, if it is disabled
     *
     * This axis type cannot be disabled by the user, but it can be enabled if
     * it is switched off.
     *
     * @param on
     * @return asynStatus
     */
    asynStatus enable(bool on);

    /**
     * @brief This axis type has an absolute encoder by default
     *
     * @return asynStatus
     */
    asynStatus readEncoderType();

    /**
     * @brief Interpret the error code and populate the user message accordingly
     *
     * @param error
     * @param userMessage
     * @param sizeUserMessage
     * @return asynStatus
     */
    asynStatus handleError(int error, char *userMessage, int sizeUserMessage);

  protected:
    turboPmacController *pC_;
};

#endif
